@@ -124,9 +124,7 @@ def main():
 
     # default the exclude list for each platform
     if win: exclude = exclude + [
-        'dos', 'dospath', 'mac', 'macpath', 'macfs', 'MACFS', 'posix',
-        'ce',
-        ]
+        'dos', 'dospath', 'mac', 'macpath', 'macfs', 'MACFS', 'posix', ]
 
     fail_import = exclude[:]
 
@@ -218,7 +216,7 @@ def main():
     ishome = os.path.exists(os.path.join(prefix, 'Python', 'ceval.c'))
 
     # locations derived from options
-    version = sys.version[:3]
+    version = '%d.%d' % sys.version_info[:2]
     flagged_version = version + sys.abiflags
     if win:
         extensions_c = 'frozen_extensions.c'
